import socket
from base_automation import report


class ClientSocket:

    def __init__(self, host='127.0.0.1', port=8080, buffer=2048):
        self._host = host
        self._port = port
        self._buffer = buffer
        self._client_socket = self.__connect_to_server()

    @report.utils.step('create server socket_connections')
    def __connect_to_server(self):
        try:
            return socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        except Exception as e:
            print(e)
            assert False

    @report.utils.step('send message to server')
    def send_message(self, messages: list):
        self._client_socket.connect((self._host, self._port))
        for x in messages:
            self._client_socket.sendall(x)
            data = self._client_socket.recv(self._buffer)
            print('Received', repr(data))

    @report.utils.step('close socket_connections')
    def close_socket(self):
        self._client_socket.close()
        print("client socket_connections closed")



import ast
import json
import os
from base_automation import report


# ---------------------------- terminal ------------------------------------#

@report.utils.step('send terminal command: {command}')
def terminal_command(command):
    try:
        step_data(f"send command to terminal:\n{command}")
        return os.system(command)
    except Exception as e:
        step_data(e)


# ---------------------------- environment ------------------------------------#

@report.utils.step("get environment items")
def get_environment_items(key):
    return os.environ.items()


@report.utils.step("get environment variable: {key}")
def get_environment_variable(key):
    return os.environ.get(key)


@report.utils.step("set environment variable: {key}, {value}")
def set_environment_variable(key, value):
    os.environ.setdefault(key, value)


# ---------------------------- report data ------------------------------------#

@report.utils.step('{step_description}')
def step_data(step_description):
    pass


@report.utils.step("assert validation - {step_description}")
def compare_data(first_condition, second_condition, step_description=None, positive_test=True):
    if positive_test:
        assert first_condition == second_condition
    else:
        assert first_condition != second_condition


# ---------------------------- files actions ------------------------------------#

@report.utils.step("dict to json")
def dict_to_json(string_content):
    return json.dumps(str_to_dict(string_content))


@report.utils.step("str to dict")
def str_to_dict(string_content):
    return ast.literal_eval(str(string_content))


@report.utils.step("load json")
def load_json(json_content):
    return json.loads(json_content)


@report.utils.step("create temp json")
def create_temp_json(file_path, data):
    json_file = open(file_path, "w")
    json_file.write(data)
    json_file.close()



import selectors
import socket
import types
from base_automation import report


class MultiServer:

    def __init__(self, host='127.0.0.1', port=8080, buffer=2048):
        self._host = host
        self._port = port
        self._buffer = buffer
        self._server_socket = self.__create_server()
        self._selectors = selectors.DefaultSelector()

    @report.utils.step('create server socket_connections')
    def __create_server(self):
        try:
            return socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        except Exception as e:
            print(e)
            assert False

    @report.utils.step('accept wrapper')
    def accept_wrapper(self, sock):
        conn, address = sock.accept()  # Should be ready to read
        print('\n\naccepted connection from', address)
        conn.setblocking(False)
        data = types.SimpleNamespace(addr=address, inb=b'', outb=b'')
        events = selectors.EVENT_READ | selectors.EVENT_WRITE
        self._selectors.register(conn, events, data=data)

    @report.utils.step('service connection')
    def service_connection(self, key, mask):
        sock = key.fileobj
        data = key.data
        if mask & selectors.EVENT_READ:
            receive_data = sock.recv(self._buffer)  # Should be ready to read
            if receive_data:
                data.outb += receive_data
            else:
                print('closing connection to', data.addr)
                self._selectors.unregister(sock)
                sock.close()
        if mask & selectors.EVENT_WRITE:
            if data.outb:
                print('echoing', repr(data.outb), 'to', data.addr)
                sent = sock.send(data.outb)  # Should be ready to write
                data.outb = data.outb[sent:]

    @report.utils.step('listen to client')
    def listen_to_client(self):
        print("Socket Started")
        self._server_socket.bind((self._host, self._port))
        self._server_socket.listen()
        print('listening on', (self._host, self._port))
        self._server_socket.setblocking(False)
        self._selectors.register(self._server_socket, selectors.EVENT_READ, data=None)

        while True:
            events = self._selectors.select(timeout=None)
            for key, mask in events:
                if key.data is None:
                    self.accept_wrapper(key.fileobj)
                else:
                    self.service_connection(key, mask)

    @report.utils.step('close server socket_connections')
    def close_socket(self):
        self._server_socket.close()
        print("server socket_connections closed")



from base_automation.socket_connections.echo_client import ClientSocket
from base_automation.socket_connections.echo_server import ServerSocket
from base_automation.socket_connections.server_multi_connections import MultiServer

__all__ = [
    'ClientSocket',
    'ServerSocket',
    'MultiServer'
]



import os
import shutil
from base_automation import report
from base_automation.utilities import shared_utilities


@report.utils.step('delete dist directory')
def delete_dist_dir(dist_dir):
    if os.path.exists(dist_dir) and os.path.isdir(dist_dir):
        shutil.rmtree(dist_dir)
    else:
        print("The dist directory does not exist")


@report.utils.step("build")
def build():
    build_commands = ["python setup.py develop",
                      "python -m pip install --upgrade pip",
                      "python -m pip install --upgrade build", "python -m build", "python setup.py sdist bdist_wheel"]
    for command in build_commands:
        shared_utilities.terminal_command(command)


@report.utils.step("upload artifact to {azure_feed_name}")
def upload_azure_artifact(azure_feed_name):
    command = f"twine upload -r {azure_feed_name} dist/*"
    shared_utilities.terminal_command(command)


@report.utils.step("upload pypi artifact")
def upload_pypi_artifact():
    command = f"twine upload -u {shared_utilities.get_environment_variable('pypi-user')} -p {shared_utilities.get_environment_variable('pypi-password')} --repository-url https://upload.pypi.org/legacy/ dist/*"
    shared_utilities.terminal_command(command)


@report.utils.feature('Build & Upload New Artifact To Azure')  # A sub-function function at large
@report.utils.severity("normal")  # Priority of test cases - 'blocker', 'critical', 'normal', 'minor', 'trivial'
def run_process(dist_dir, azure_feeds: list = None, azure_artifact: bool = False, pypi_artifact: bool = False):
    delete_dist_dir(dist_dir)
    build()
    if pypi_artifact:
        upload_pypi_artifact()
    if azure_artifact:
        for feed in azure_feeds:
            upload_azure_artifact(feed)



from base_automation import report
from base_automation.utilities import shared_utilities
from base_automation import sql
from base_automation import socket
from base_automation import upload_artifact
from base_automation.schema import schema

__all__ = [
    'report',
    'sql',
    'shared_utilities',
    'socket',
    'schema',
    'upload_artifact'
]



import mysql.connector
from base_automation import report


class MySqlServer:

    @report.utils.step("initiate my sql_capabilities server connection")
    def __init__(self, user='root', password='password', host='127.0.0.1', database='sys'):
        self._connection = mysql.connector.connect(user=user, password=password,
                                                   host=host,
                                                   database=database)
        self._cursor = self._connection.cursor()
        self._data_result = None

    @report.utils.step("get data")
    def get_data(self, query, close_connection=True):
        try:
            self._cursor.execute(query)
            self._data_result = list(self._cursor.fetchall())
            if close_connection:
                self.close_connection()
            return self._data_result
        except Exception as e:
            print(e)
            self.close_connection()
            assert False

    @report.utils.step("insert data")
    def insert_data(self, query, data, is_tuple=True, close_connection=True):
        try:
            if is_tuple:
                self._cursor.executemany(query, data)
            else:
                self._cursor.execute(query, data)

            self._connection.commit()
            if close_connection:
                self.close_connection()
        except Exception as e:
            print(e)
            self.close_connection()
            assert False

    @report.utils.step("close connection")
    def close_connection(self):
        try:
            self._connection.close()
        except Exception as e:
            print(e)
            assert False



import os
import json
from base_automation import report
from base_automation.utilities import shared_utilities
from genson import SchemaBuilder
from jsonschema import validate


class JsonSchema:

    def __init__(self):
        self._builder = SchemaBuilder()
        self._json_instance = None
        self._schema = None
        self._temp_schema = None
        self._result = None

    @report.utils.step("create schema")
    def initiate_schema(self, file_path=None, json_content_is_exists=None, save_file_name_path=None):
        if file_path is not None:
            with open(file_path, 'r') as file:
                self._json_instance = shared_utilities.load_json(file.read())
        else:
            self._json_instance = json.dumps(json_content_is_exists)

        if save_file_name_path is not None and not os.path.exists(save_file_name_path):
            with open(save_file_name_path, 'w') as file:
                self._builder.add_object(self._json_instance)
                self._temp_schema = self._builder.to_schema()
                file.write(json.dumps(self._temp_schema))
                file.close()

        self.__read_schema(file_path=save_file_name_path, schema_content=json_content_is_exists)

    @report.utils.step("read schema")
    def __read_schema(self, file_path=None, schema_content=None):
        if file_path is not None:
            with open(file_path, 'r') as file:
                self._schema = shared_utilities.load_json(file.read())

        else:
            self._schema = schema_content

    @report.utils.step("validate schema")
    def validate_schema(self):
        try:
            self._result = validate(instance=self._json_instance, schema=self._schema)
            shared_utilities.step_data("The schema test finish with success")
        except Exception as e:
            shared_utilities.step_data(f"Exception:\n{e.__str__()}")
            assert False



import os
from setuptools import setup, find_packages

lib_folder = os.path.dirname(os.path.realpath(__file__))
requirement_file = 'requirements.txt'
full_path = '/'.join([lib_folder, requirement_file])

install_requires = []
if os.path.isfile(full_path):
    with open(full_path) as f:
        install_requires = f.read().splitlines()

with open("README.md", "r") as fh:
    long_description = fh.read()


setup(name='base-automation',
      version='0.0.3',
      description='base-automation',
      url='https://upload.pypi.org/legacy/',
      author='Automation Team',
      long_description=long_description,
      long_description_content_type="text/markdown",
      author_email='automation.team@rafael.co.il',
      license='MIT',
      install_requires=install_requires,
      packages=find_packages(),
      zip_safe=False,
      classifiers=[
          "Programming Language :: Python :: 3",
          "License :: OSI Approved :: MIT License",
          "Operating System :: OS Independent",
      ],
      python_requires='>=3.6',
      )



import socket
from base_automation import report


class ServerSocket:

    def __init__(self, host='127.0.0.1', port=8080, buffer=2048):
        self._host = host
        self._port = port
        self._buffer = buffer
        self._data = None
        self._server_socket = self.__create_server()

    @report.utils.step('create server socket_connections')
    def __create_server(self):
        try:
            return socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        except Exception as e:
            print(e)
            assert False

    @report.utils.step('listen to client')
    def listen_to_client(self):
        print("Socket Started")
        self._server_socket.bind((self._host, self._port))
        self._server_socket.listen()
        print('listening on', (self._host, self._port))
        conn, address = self._server_socket.accept()
        with conn:
            print('Connected by', address)
            while True:
                self._data = conn.recv(self._buffer)
                if not self._data:
                    break

                conn.sendall(self._data)
                print(self._data)

    @report.utils.step('close server socket_connections')
    def close_socket(self):
        self._server_socket.close()
        print("server socket_connections closed")

    @report.utils.step('str to bytes')
    def __str_to_bytes(self):
        self._data = str.encode(self._data)
        print(type(self._data))  # ensure it is byte representation
